unit Biglist;

interface

Uses
    Sysutils, Classes;

type


 TCompareResult = (crLessThan, crEqual, crGreaterThan);
 TCompareEvent = Procedure (ItemToInsert, Key : TObject;
                 Var Index : TCompareResult; Var PartialResult : Boolean) of Object;

 TBigList  = Class(TObject)
             private
                FDuplicates  : TDuplicates;
                FSorted      : Boolean;
                FOnCompare   : TCompareEvent;
                FLastSortedItem: TObject;
                FLastMax,
                FLastMin     : Longint;
             protected

                FList        : TList;
                FChildlist   : TBigList;


                Function CreateInstance : TBigList; Virtual;

                procedure GiveItemToChild(ItemToGive: Tobject);
                procedure SetFSorted(ThisState : Boolean);
                { Write access method for the sorted property }
                Function ReturnCount : LonGint;
                procedure GiveItemDirectToChild(Index : Longint; ItemToGive: Tobject);
                procedure InsertDirect(Index : LongInt; ThisObject : TObject);


             Public
                Function At(ThisIndex : Longint) : TObject;
                { Returns the object at the specified position }
                Function Add(ThisObject : TObject) : LongInt;
                { Adds the given object to the list }
                Function AtInsert(Index : Longint; ThisObject : TObject) : Longint;
                { Inserts the object at the specified location. If sorted then the object }
                { is ADDed at the correct position and NOT at Index }
                Function AtDelete(Index : LongInt) : Integer;
                { Deletes the object at the specified location }
                Function IndexOf(ThisObject : TObject) : Longint;
                { Returns the index of the given object. (-1 if the object is not present within the list) }
                Procedure Clear; Virtual;

                Constructor Create;
                Destructor Destroy; Override;

                Property Sorted       : Boolean Read FSorted Write SetFSorted;
                Property OnCompare    : TCompareEvent Read FOnCompare Write FOnCompare;
                Property Duplicates   : TDuplicates Read FDuplicates Write FDuplicates;
                Property List         : TList Read FList;
                Property NextList     : TBigList Read FChildList;
                Property ObjectAt[Index : LongInt] : TObject Read At; Default;
                property Count        : Longint Read ReturnCount;
                { Returns the number of items within the list }

             End;


implementation

Const
  MaxCollectionSize = 8000;
  MaxItems = (MaxCollectionSize div 2) - 100;

Function TBigList.At(ThisIndex : Longint) : TObject;
begin
   Result := Nil;
   If ThisIndex < Flist.Count then
      Result := FList.Items[Thisindex]
   Else
      { Index cannot be found within main list. Reduce index count and pass it }
      { on to it's child list (if one is present)                              }
      if Assigned(FChildList) then
      begin
         ThisIndex := ThisIndex - Flist.Count;
         Result := FChildList.At(ThisIndex);
      End;
End;

procedure TBigList.GiveItemToChild(ItemToGive: Tobject);
begin
  { create subsidary list }
   If Not Assigned(FChildList) then
   Begin
      FChildList := CreateInstance;
      FChildList.OnCompare := OnCompare;
      FChildList.Sorted := Sorted;
      FChildList.Duplicates := Duplicates;

   End;
   { pass item on to this list }
   FChildList.Add(ItemToGive);
End;


Function TBigList.CreateInstance : TBigList;
{ Creates an instance of the child list }
Begin
   Result := TBigList.Create;
End;



Function TBigList.Add(ThisObject : TObject) : LongInt;
var
   Index         : LongInt;
   CompareResult : TCompareResult;

   Function FindPosition : Longint;
   { Determine the position the object should take using a binary chop method }
   var
      Min,Max, Middle : LongInt;
      CompareResult   : TCompareResult;
      Stop            : Boolean;
      PartialResult   : Boolean;
   begin
      Stop := False;
      PartialResult := False;
      Min := 0;

      Max := Count - 1;

      { Try Last item sorted first as a starting point, rather than the middle list entry }

      Middle := IndexOf(FLastSortedItem);
      If (Middle < Count) and (Middle >= 0) then
      begin
         CompareResult := crEqual;
         OnCompare(ThisObject, FLastSortedItem, CompareResult, partialresult);
         If CompareResult = crGreaterthan then
         begin
            Min := Middle;
            {----------------------------------------------------------------------------}
            (*WBF10 SJW                                                                  }
             Middle := Middle + 1;}
            {---------------------------------------------------------------------------*)

            If Middle = (Count - 1) then
            begin
               Stop := True;
               Middle := Count;
            End;
         End
         Else if CompareResult = crLessthan then
         Begin
            Max := Middle;
            If Middle = 0 then
               Stop := true
         End
         Else If CompareResult = crEqual then
            Stop := True;

(*         If PartialResult and (FLastmax >= 0) and (FLastMin >= 0) then
         Begin
            Min :=FLastMin;
            max := Flastmax;
         End; *)
      End
      Else
      Begin
         Middle := 0;

         Stop := Min > Max;
      End;


      FlastMax := -1;
      FLastMin := -1;
      PartialResult := False;

      { If first search not successful }
      If Not Stop then
      Begin
         While (Not Stop) do
         Begin
            Middle := ((Max - Min) div 2) + Min;
            CompareResult := crEqual;

            OnCompare(ThisObject, At(Middle), CompareResult, PartialResult);

            If (Min >= Max) or (Max <= Min)  then
               Stop := True;

            If CompareResult = crGreaterthan then
            begin
               If Min <> Middle then
                  Min := Middle
               Else
                  Min := Max;
               { Add item to end of list }
               If Middle = (Count - 1) then
               begin
                  Stop := True;
                  Middle := Count;
               End;
            End
            Else if CompareResult = crLessthan then
            Begin
               Max := Middle;
               If Middle = 0 then
                  Stop := true;
            End
            Else if CompareResult = crEqual then
               Stop := True;

            If PartialResult and (Flastmax < 0) then
            Begin
               Flastmax := max;
               Flastmin := Min;
            End;
         End;
      End;
      FLastSortedItem := ThisObject;
      FindPosition := Middle;
   End;

Begin
   Result := 0;
   CompareResult := crLessThan;

   if (Not Sorted) or (not Assigned(FOnCompare)) then
   begin
      { is there enough room to add the new item }
      If (Flist.Count < MaxItems) then
         FList.Add(ThisObject)  //Insert(ThisObject)
      Else { otherwise pass the value on to the child list }
         GiveItemToChild(Thisobject);
   End
   Else
   Begin
       Index := FindPosition;
       { ignore duplicates ? }
       If (Duplicates = dupAccept) or (CompareResult <> crEqual) then
          { has a place been found for this item  ? }
          InsertDirect(Index, ThisObject);
   End;
End;


procedure TBigList.GiveItemDirectToChild(Index : Longint; ItemToGive: Tobject);
begin
  { create subsidary list }
   If Not Assigned(FChildList) then
   Begin
      FChildList := CreateInstance;
      FChildList.OnCompare := OnCompare;
      FChildList.Sorted := Sorted;
      FChildList.Duplicates := Duplicates;

   End;
   { pass item on to this list }
   FChildList.InsertDirect(Index, ItemToGive);
End;


procedure TBigList.InsertDirect(Index : LongInt; ThisObject : TObject);
Begin
  {----------------------------------------------------------------------------}
  { Index specified lies within local list ?                                   }
  {----------------------------------------------------------------------------}
   If (Index < FList.Count) or (FList.Count = 0) or (FList.Count < MaxItems) then
   begin
      {----------------------------------------------------------------------------}
      { if list full then move last item into the child's list                     }
      {----------------------------------------------------------------------------}
      If FList.Count = MaxItems then
      begin
         GiveItemDirectToChild(0,FList.Items[FList.Count - 1]);
         FList.Delete(FList.Count - 1);
      End;
      { insert item }
      FList.Insert(Index, ThisObject);
   End
   Else
      {----------------------------------------------------------------------------}
      { Pass item directly on to child, reducing index by local list count         }
      {----------------------------------------------------------------------------}
      GiveItemDirectToChild(Index - FList.Count, ThisObject);
End;


Function TBigList.IndexOf(ThisObject : TObject) : Longint;
{ Returns the index of the given object. (-1 if the object is not present within the list) }
Begin
    Result := FList.IndexOf(ThisObject);
    If (Result < 0) and (Assigned(FChildList)) then
    Begin
       Result := FChildlist.Indexof(ThisObject);
       If Result >= 0 then
          Result := Result + FList.Count;
    End;
End;



Function TBigList.AtInsert(Index : Longint; ThisObject : TObject) : Longint;
begin
   result := 0;
   If Count <= Index then
   begin
      If Sorted then
         Add(ThisObject)
      Else
      begin
         { is there enough room to add the new item }
         If Flist.Count < MaxItems then
            FList.Insert(Index, ThisObject)
         Else { otherwise pass the value on to the child list }
         begin
            { reduce the insert position to take into account number of items skipped in this list }
            Dec(Index, Flist.Count);
            { instruct the child to add it }
            If Assigned(FChildList) then
               FChildList.AtInsert(Index, ThisObject)
            Else
               Raise Exception.Create('Instructed to ATINSERT at an invalid list position.');
         End;
      End;
   End
   Else
     Raise Exception.Create('Instructed to ATINSERT at an invalid list position.');
End;


Function TBigList.AtDelete(Index : LongInt) : Integer;
var
   TempObj : TObject;
begin
   Result := 0;
   If Index < FList.Count then
   begin
      FList.Delete(Index);
      { if there is a subsidary list, then take the first element from this }
      { and place it at the end of this list }
      If Assigned(FChildList) and (FChildList.Count > 0) then
      begin
         Tempobj := FChildList.At(0);
         FChildList.AtDelete(0);
         InsertDirect(FList.Count, TempObj);
      End;
   End
   Else
   begin
      Dec(Index, FList.Count);
      If Assigned(FChildList) then
         FChildList.AtDelete(Index)
      Else Raise Exception.Create('Instructed to DELETE object at invalid list position.');
   End;
   If Assigned(FChildlist) then
      If FChildList.Count = 0 then
      Begin
          FChildList.Free;
          FChildList := Nil;
      End;
End;



Function TBigList.ReturnCount : Longint;
Begin
   Result := FList.Count;
   If Assigned(FChildList) then
      Result := Result + FChildList.Count;
End;


procedure TBigList.Clear;
{ Clears all items. (Count = 0) }
begin
   FList.Clear;

   If Assigned(FChildList) then
      FChildList.Free;
End;



procedure TBigList.SetFSorted(ThisState : Boolean);
Var
   Templist : TBigList;
begin
    If FSorted <> ThisState then
    begin
       FSorted := ThisState;
       If Sorted and (Count > 0) then
       begin
          TempList := CreateInstance;
          Templist.Sorted := True;
          TempList.Duplicates := Duplicates;
          TempList.OnCompare := OnCompare;
          { transfer objects into temporary storage, sorting at the same time }
          While Count > 0 do
          begin
             TempList.Add(At(0));
             AtDelete(0);
          End;
          { temporarily turn sorted off, and move objects back into main list. }
          { The objects in temp list will already be sorted }
          FSorted := False;
          If Assigned(FChildList) then
             FChildList.Sorted := False;

          While TempList.Count > 0 do
          begin
             Add(TempList.At(0));
             TempList.AtDelete(0);
          End;
          { restore sorted state }
          FSorted := true;
          If Assigned(FChildList) then
             FChildList.Sorted := True;

          { free temporary list }
          Templist.Free;
       End;
    End;
End;

Constructor TBigList.Create;
Begin
   Inherited Create;
   FList := TList.create;
   { initialise the pointer to the child list -}
   { this will hold all elements which the main list cannot hold}
   FChildList := Nil;
   FSorted := False;
   FDuplicates := dupIgnore;
   FOnCompare := Nil;
   FLastSortedItem := Nil;
   FLastMax := -1;
   FLastMin := -1;
End;


Destructor TBigList.Destroy;
begin
   If Assigned(FList) then
      FList.Free;
   Flist := Nil;

   { Now free the child list }
   if Assigned(FChildList) then
      FChildList.Free;
   Inherited Destroy;
End;


end.
