{-----------------------------------------------------------------------}
{     UNIT       :   GP_UTIL                                            }
{                                                                       }
{     EXPLANATION:   DEFINES THE UTILITY ROUTINES THAT ARE AVAILABLE    }
{                                                                       }
{     AUTHOR     :   S.WEST                                             }
{                                                                       }
{     DATE       :   21.12.95                                           }
{                                                                       }
{-----------------------------------------------------------------------}
{ Note that any amendments/additions surrounded by CMW comments are     }
{ made by C.Waller                                                      }
{-----------------------------------------------------------------------}

unit GP_util;

interface

Uses
  dialogs,
  Controls,
  StdCtrls,
  Graphics,
  WinTypes,
  Forms,
  SysUtils,
  Menus,
  Grids,
  Winprocs,
  ExtCtrls;

function GP_Real2Str(ThisReal : real; Decimals : Byte):string;
Function GP_Str2Real(ThisStr : String) : Real;

Function GP_RemoveSpaces(ThisString : String) : String;
function GP_Max(a, b :longint) :longint;
function GP_Min(a, b :longint) :longint;

implementation

function GP_FileExists(FileName: String): Boolean;
{ Boolean function that returns True if the file exists;otherwise,
 it returns False. Closes the file if it exists. }
var
 F: file;
begin
 {$I-}
 Assign(F, FileName);
 Reset(F);
 Close(F);
 {$I+}
 GP_FileExists := (IOResult = 0)and (FileName <> '');
end;  { FileExists }


function GP_Real2Str(ThisReal : real; Decimals : Byte):string;
{ Converts real to a string. If Decimals = 0 then the Real is Rounded }
var thestring : string;

begin
   If Decimals = 0 then
      Result := IntToStr(Round(ThisReal))
   Else
   Begin
      Str(ThisReal:11:decimals,thestring);
      GP_Real2Str := GP_RemoveSpaces(thestring);
   End;
end;

Function GP_Str2Real(ThisStr : String) : Real;
{ Returns the numerical value of the given string. ZERO is returned if the string does not }
{ represent a valid numerical entry }
Var
    ThisNum   : Real;
    Error     : Integer;
begin
    Result := 0;
    If ThisStr <> '' then
    Begin
        Val(GP_RemoveSpaces(ThisStr), ThisNum, Error);
        If (Error <> 0)then { invalid entry }
           Result := 0
        Else
           Result := ThisNum;
    End;
End;

Function GP_RemoveSpaces(ThisString : String) : String;
{ Removes all the spaces from the given string }
Var
   Counter : Integer;
   Stop    : Boolean;
begin
   Stop := False;
   Counter := 1;

   While Not Stop do
   Begin
       If Counter > Length(ThisString) then { exceeded length of line ?}
          Stop := True
       else
       Begin
          If (ThisString[Counter] = ' ') then
          Begin
             ThisString := Copy(ThisString, 1, Counter - 1) + Copy(ThisString, Counter + 1, length(ThisString) - Counter);
             Dec(Counter);
          end;
       end;
       Inc(Counter);
   end;
{   If ThisString[Length(ThisString)] = '.' then
      ThisString := ThisString + '00';}
   GP_RemoveSpaces := ThisString;
end;

function GP_Min(a, b :longint) :longint;
{ returns lowest of the two values }
begin
  if (a < b) then
    result := a
  else
    result := b;
end;

function GP_Max(a, b :longint) :longint;
{ returns highest of the two values }
begin
  if (a > b) then
    result := a
  else
    result := b;
end;

end.
