unit Rep_opts;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls, Buttons, PRN_Type;

type
  TExportOptionsDialog = class(TForm)
    PrintingOptionsPanel: TPanel;
    GridLinesChkBox: TCheckBox;
    PrintHdrChkBox: TCheckBox;          
    PrintDescChkBox: TCheckBox;
    PageSpreadChkBox: TCheckBox;
    Panel1: TPanel;
    PrintButton: TBitBtn;
    CancelButton: TBitBtn;
    ExportButton: TBitBtn;
    ExportOptionsPanel: TPanel;
    GroupBox1: TGroupBox;
    DelimRadio: TRadioButton;
    FixedRadio: TRadioButton;
    XLRadio: TRadioButton;
    Label1: TLabel;
    DelimiterEditBox: TEdit;
    procedure DelimRadioClick(Sender: TObject);
    procedure DelimiterEditBoxChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FContext :PRN_TOptionsDialogContext;
    FExportType: PRN_TExportType;
    FDelimiter: char;
    FAddSpace: Boolean;
  public
    constructor create(AOwner :TComponent; context :PRN_TOptionsDialogContext);
    property ExportType :PRN_TExportType  read FExportType  write FExportType;
    property Delimiter :char             read FDelimiter   write FDelimiter;
    property AddSpace :Boolean           read FAddSpace    write FAddSpace;
  end;

var
  ExportOptionsDialog: TExportOptionsDialog;

implementation

{$R *.DFM}

uses
  GP_Util;

constructor TExportOptionsDialog.create(AOwner :TComponent; Context :PRN_TOptionsDialogContext);
begin
  inherited create(AOwner);

  FContext := context;

  { position the cancel button - vertically centered, 5 pix from right }
  CancelButton.Left := Panel1.Width - CancelButton.Width - 5;
  CancelButton.Top := round((Panel1.Height - CancelButton.Height) / 2);

  if (FContext = dcExportOptions) then
  begin
    { we can get rid of this }
    PrintingOptionsPanel.Free;
    PrintButton.visible := False;

    { set up components at run time }
    caption := 'Export Options';
    ExportButton.visible := True;
    ExportButton.default := True;
    ExportButton.Left := CancelButton.Left - ExportButton.Width - 5;
    ExportButton.Top := Cancelbutton.Top;
  end

  else if (FContext = dcPrintOptions) then
  begin
    { we can get rid of this }
    ExportOptionsPanel.Free;
    ExportButton.visible := False;

    { set up components at run time }
    caption := 'Print Options';
    PrintButton.visible := True;
    PrintButton.default := True;
    PrintButton.Left := CancelButton.Left - ExportButton.Width - 5;
    PrintButton.Top := Cancelbutton.Top;
  end;
end;

procedure TExportOptionsDialog.DelimRadioClick(Sender: TObject);
begin
  if (Sender = XLRadio) then
  begin
    ExportType := etExcel;
    DelimiterEditBox.enabled := False;
    Label1.enabled := False;
  end
  else
  if (Sender = DelimRadio) then
  begin
    ExportType:=etDelimited;

    DelimiterEditBox.enabled := True;
    Label1.enabled := True;
  end
  else
  if (Sender = FixedRadio) then
  begin
    ExportType:=etFixed;

    DelimiterEditBox.enabled:=False;
    Label1.enabled:=False;
  end;
end;

procedure TExportOptionsDialog.DelimiterEditBoxChange(Sender: TObject);
begin
  { set the delimiting character property }
  if (Length(DelimiterEditBox.Text) > 0) then
    Delimiter := DelimiterEditBox.Text[1]
  else
    Delimiter := ' ';
end;

procedure TExportOptionsDialog.FormCreate(Sender: TObject);
begin
  if (FContext = dcExportOptions) then
  begin
    ExportType := etExcel;
    DelimiterEditBox.enabled := False;
    Label1.enabled := False;
    ActiveControl := XLRadio;
  end
  else
    ActiveControl := PrintingOptionsPanel;
end;

end.
