 {-----------------------------------------------------------------------}
{     UNIT       :   PrefsDlg                                           }
{                                                                       }
{     EXPLANATION:   Defines the preferences dialog                     }
{                                                                       }
{     AUTHOR     :   P.Chapman                                          }
{                                                                       }
{     DATE       :   16/06/97                                           }
{                                                                       }
{-----------------------------------------------------------------------}
{ Modifications:                                                        }
{                                                                       }
{ W5 16/06/97.PAC - Unit Created.                                       }
{-----------------------------------------------------------------------}
unit PrefsDlg;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, TabNotBk, Buttons, ExtCtrls, DBCtrls, Spin,
  Printers, ComCtrls;

type

  TPreferencesDlg = class(TForm)
    pnButtonPanel: TPanel;
    CloseButton: TBitBtn;
    nbNoteBook: TTabbedNotebook;
    gbMargins: TGroupBox;
    seHorzMargin: TSpinEdit;
    seVertMargin: TSpinEdit;
    Label1: TLabel;
    Label2: TLabel;
    gbAppearance: TGroupBox;
    cbShowHeader: TCheckBox;                                
    cbShowFooter: TCheckBox;
    cbShowGridLines: TCheckBox;
    cbFitToPage: TCheckBox;
    gbPreview: TGroupBox;
    pnPreview: TPanel;
    pbPreview: TPaintBox;
    cbShowDescription: TCheckBox;
    Label3: TLabel;
    Label4: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure AppearanceCheckBoxClick(Sender: TObject);
    procedure seMarginChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure pbPreviewPaint(Sender: TObject);
  private
    { Private declarations }
    FFitToPage,
    FShowGridLines,
    FShowHeader,
    FShowFooter,
    FShowDescription :Boolean;

    FUsablePageWidth,
    FUsablePageHeight :longint;

    FVPrinterToScreenRatio,
    FHPrinterToScreenRatio :real;

    FBitmap :TBitmap;
    FPbInitialising :boolean;

    procedure SetPaintBoxDimensions;
    procedure RefreshPaintBox;

  public
    { Public declarations }
    constructor create(AOwner :TComponent); override;
  end;

var
  PreferencesDlg: TPreferencesDlg;

implementation

uses
  GP_util, PRN_Type;

{$R *.DFM}

const
  MetricToInchesRatio = 0.0254;
  InchesToMetricRatio = 1 / MetricToInchesRatio;
  ReportColCount = 10;

constructor TPreferencesDlg.Create(AOwner :TComponent);
begin
  FShowGridLines := True;
  FShowHeader := True;
  FShowDescription := True;
  FShowFooter := True;
  FFitToPage := False;

  { create an offscreen bitmap the size of the preview paintbox }
  FBitmap := TBitmap.create;

  FPbInitialising := True;
  inherited create(AOwner);
  FPbInitialising := False;

  RefreshPaintBox;
end;

procedure TPreferencesDlg.FormCreate(Sender: TObject);
begin
//  GP_SetFont(Font, Screen);

  pbPreview.Font.Assign(Font);
  pbPreview.Canvas.Font.Assign(pbPreview.Font);
  pbPreview.Canvas.Font.Size := 4;

  FBitmap.Canvas.Font.Assign(pbPreview.Canvas.Font);

  { set up the appearances checkboxes }
  cbShowGridLines.Checked := FShowGridLines;
  cbShowHeader.Checked := FShowHeader;
  cbShowDescription.Checked := FShowDescription;
  cbShowFooter.Checked := FShowFooter;
  cbFitToPage.Checked := FFitToPage;

  { scale the paintbox size to mimic the dimensions of }
  { the printers paper size }
  SetPaintBoxDimensions;

  { set the bitmaps size large enough to hold all of the paintbox surface }
  FBitmap.Width := pbPreview.Width;
  FBitmap.Height := pbPreview.Height;

  { set some scaling ratio fields }
  FVPrinterToScreenRatio := Printer.PageHeight / PbPreview.ClientHeight;
  FHPrinterToScreenRatio := Printer.PageWidth / pbPreview.ClientWidth;

  { set the min and max margin values, at least half the page width and height }
  { must be printable }
  with seHorzMargin do
  begin
    MinValue := 0;
    MaxValue := (Printer.PageWidth div 4);
  end;  

  with seVertMargin do
  begin
    MinValue := 0;
    MaxValue := (Printer.PageHeight div 4);
  end;

  { set how much width and height of the paintbox we can print on }
  { according to the values set for the margins }
  FUsablePageWidth := pbPreview.ClientWidth -
                    Round(2 * (seHorzMargin.Value / FHPrinterToScreenRatio));
  FUsablePageHeight := pbPreview.ClientHeight -
                     Round(2 * (seVertMargin.Value / FVPrinterToScreenRatio));
end;

procedure TPreferencesDlg.RefreshPaintBox;
var
  YCursor :longint;
  HeaderText, FooterText :string;
  HeaderRect, FooterRect :TRect;
  iCol, iRow :integer;
  CellRect :TRect;
  ColWidth, RowHeight :integer;
  CellText :string;
  FooterHeight :longint;
begin
  if not FPbInitialising then
  begin
    with FBitmap, Canvas do
    begin
      FBitmap.Canvas.Font.Size := 6;
      YCursor := Round(seVertMargin.Value / FVPrinterToScreenRatio);

      { clear the paintbox }
      Brush.Style := bsSolid;
      Brush.Color := clWhite;
      FillRect(Rect(0, 0, Width, Height));

      { draw the header if appropriate }
      if FShowHeader then
      begin
        Font.Color := clBlack;
        Brush.Style := bsClear;
        HeaderRect := Rect(Round(seHorzMargin.Value / FHPrinterToScreenRatio), YCursor,
                           FBitmap.Width - Round(seHorzMargin.Value / FHPrinterToScreenRatio), YCursor + 1);
        HeaderText := 'Report Header';

        { draw a centred header }
        DrawText(FBitmap.Canvas.Handle, @HeaderText[1], Length(HeaderText), HeaderRect,
                  DT_CALCRECT or DT_VCENTER or DT_CENTER);

        HeaderRect.Right := Width - Round(seHorzMargin.Value / FHPrinterToScreenRatio);
        DrawText(FBitmap.Canvas.Handle, @HeaderText[1], Length(HeaderText), HeaderRect,
                  DT_VCENTER or DT_CENTER);

        Brush.Color := clGray;
        Brush.Style := bsSolid;
        FrameRect(Rect(Round(seHorzMargin.Value / FHPrinterToScreenRatio), YCursor,
                        Width - Round(seHorzMargin.Value / FHPrinterToScreenRatio), HeaderRect.Bottom));

        YCursor := HeaderRect.Bottom + 2;
      end;

      { draw the description if required }
      if FShowDescription then
      begin
        Brush.Style := bsClear;
        HeaderRect := Rect(Round(seHorzMargin.Value / FHPrinterToScreenRatio), YCursor,
                      Width - Round(seHorzMargin.Value / FHPrinterToScreenRatio), YCursor + 1);
        HeaderText := 'Report Description';

        { draw a centred header }
        DrawText(FBitmap.Canvas.Handle, @HeaderText[1], Length(HeaderText), HeaderRect,
                  DT_CALCRECT or DT_LEFT or DT_WORDBREAK);

        HeaderRect.Right := Width - Round(seHorzMargin.Value / FHPrinterToScreenRatio);
        DrawText(FBitmap.Canvas.Handle, @HeaderText[1], Length(HeaderText), HeaderRect,
                  DT_LEFT or DT_WORDBREAK);

        YCursor := HeaderRect.Bottom + 2;
      end;

      { draw the footer if appropriate }
      if FShowFooter then
      begin
        Font.Size := 6;
        Brush.Style := bsClear;
        FooterRect := Rect(Round(seHorzMargin.Value / FHPrinterToScreenRatio), 0,
                           Width - Round(seHorzMargin.Value / FHPrinterToScreenRatio), 1);
        FooterText := FormatDateTime('dd/mm/yy', Now) + ' - Page 1';

        { draw a centred header }
        DrawText(FBitmap.Canvas.Handle, @FooterText[1], Length(FooterText), FooterRect,
                  DT_CALCRECT or DT_VCENTER or DT_CENTER);

        { modify the rect so that is is aligned to page bottom }
        FooterRect.Top := Height - Round(seVertMargin.Value / FVPrinterToScreenRatio) - (FooterRect.Bottom - FooterRect.Top);
        FooterRect.Bottom := Height - Round(seVertMargin.Value / FVPrinterToScreenRatio);
        FooterRect.Right := Width - Round(seHorzMargin.Value / FHPrinterToScreenRatio);

        DrawText(FBitmap.Canvas.Handle, @FooterText[1], Length(FooterText), FooterRect,
                  DT_VCENTER or DT_CENTER);

        Brush.Color := clGray;
        Brush.Style := bsSolid;
        FrameRect(Rect(Round(seHorzMargin.Value / FHPrinterToScreenRatio), FooterRect.Top,
                  Width - Round(seHorzMargin.Value / FHPrinterToScreenRatio), FooterRect.Bottom));
        FooterHeight := FooterRect.Bottom - FooterRect.Top;
      end
      else
        FooterHeight := 0;

     { draw the report grid }
      if FFitToPage then
      begin
        { calculate the width of each column }
        ColWidth := FUsablePageWidth div ReportColCount;
        RowHeight := 5;
      end
      else
      begin
        ColWidth := 20;
        RowHeight := 10;
      end;

      Brush.Style := bsSolid;
      Brush.Color := clGray;

      for iRow := 0 to 10 do
      begin
        for iCol := 0 to ReportColCount - 1 do
        begin
          CellRect := Rect(Round(seHorzMargin.Value / FHPrinterToScreenRatio) +
                             iCol * ColWidth,
                           YCursor + (iRow * RowHeight),
                           Round(seHorzMargin.Value / FHPrinterToScreenRatio) +
                             (iCol + 1) * ColWidth + 1,
                           YCursor + ((iRow  + 1) * RowHeight) + 1);

          { see if this cellrect exceeds the right or (bottom margin - footerheight) }
          if ((CellRect.Right <= pbPreview.Width - Round(seHorzMargin.Value / FHPrinterToScreenRatio + 1)) and
              ((CellRect.Bottom <= pbPreview.Height - Round(seVertMargin.Value / FVPrinterToScreenRatio) - FooterHeight + 1)))
              then
          begin
            Font.Size := 4;
            CellText := 'x';
            if (iRow = 0) or (iCol = 0) then
            begin
              Brush.Color := clSilver;
              Brush.Style := bsSolid;
              FillRect(CellRect);
              Brush.Color := clGray;

              if (iRow = 0) then
              begin
                CellText := '[' + Chr(Ord('A') + iCol) + ']';
              end
              else
              if (iCol = 0) then
                CellText := '[' + IntToStr(iRow) + ']';
            end
            else
              CellText := IntToStr(iRow * 3 * iCol * 4);

            Brush.Style := bsClear;
            DrawText(FBitmap.Canvas.Handle, @CellText[1], Length(CellText), CellRect,
                       DT_VCENTER or DT_LEFT);
            Brush.Style := bsSolid;
            Brush.Color := clGray;

            if FShowGridLines then
              FrameRect(CellRect);
          end;
        end;
      end;
    end;
    { copy the offscreen bitmap to the canvas of the paintbox }
    pbPreview.Canvas.Draw(0, 0, FBitmap);
  end;
end;

procedure TPreferencesDlg.AppearanceCheckBoxClick(Sender: TObject);
begin
  if (Sender = cbShowGridLines) then
    FShowGridLines := (Sender as TCheckBox).Checked
  else
  if (Sender = cbShowHeader) then
    FShowHeader := (Sender as TCheckBox).Checked
  else
  if (Sender = cbShowDescription) then
    FShowDescription := (Sender as TCheckBox).Checked
  else
  if (Sender = cbShowFooter) then
    FShowFooter := (Sender as TCheckBox).Checked
  else
  if (Sender = cbFitToPage) then
    FFitToPage := (Sender as TCheckBox).Checked;

  { update the preview paintbox to reflect the option changes }
  RefreshPaintBox;
end;

procedure TPreferencesDlg.SetPaintBoxDimensions;
{ retrieves the printers paper size, and sets the paintbox size }
{ so that is has the same aspect ratio }
var
  paperheight,
  paperwidth :longint;
  aspect, pbAspect :real;
begin
  paperheight := Printer.PageHeight;
  paperwidth := Printer.PageWidth;

  { get the aspect of the paintbox }
  pbAspect := pbPreview.ClientHeight / pbPreview.ClientWidth;

  { get the aspect of the printer paper }
  aspect := paperheight / paperwidth;

  if (pbAspect > aspect) then
  begin
    pbPreview.Width := pnPreview.ClientWidth - 2;
    pbPreview.Height := Round(aspect * pbPreview.width);
  end
  else
  begin
    pbPreview.Height := pnPreview.ClientHeight - 2;
    pbPreview.Width := Round(pbPreview.Height / aspect);
  end;

  { center the paintbox horizontally }
  pbPreview.Top := (pnPreview.ClientHeight - PbPreview.Height) div 2;

  { center the paintbox vertically }
  pbPreview.Left := (pnPreview.ClientWidth - PbPreview.Width) div 2;

end;

procedure TPreferencesDlg.seMarginChange(Sender: TObject);
begin
  try
    { modify the amount of usable page area }
    if (Sender = seHorzMargin) then
    begin
      FUsablePageWidth := pbPreview.ClientWidth -
                        Round(2 * (seHorzMargin.Value / FHPrinterToScreenRatio));
    end
    else
    if (Sender = seVertMargin) then
    begin
      FUsablePageHeight := pbPreview.ClientHeight -
                         Round(2 * (seVertMargin.Value / FVPrinterToScreenRatio));
    end;

    RefreshPaintBox;
  except
    on E: Exception do
      showmessage('l');
  end;      
end;



procedure TPreferencesDlg.FormDestroy(Sender: TObject);
begin
  if assigned(FBitmap) then
    FBitmap.Free;
  FBitmap := nil;  
end;

procedure TPreferencesDlg.pbPreviewPaint(Sender: TObject);
begin
  { copy the offscreen bitmap to the canvas of the paintbox }
  pbPreview.Canvas.Draw(0, 0, FBitmap);
end;

end.
